import math
import os
import tempfile

import pytest
import spiceypy as sp
from spiceypy.utils.exceptions import NotFoundError

from cesium_script.czml.util import (
    generate_cartesians,
    generate_html_table,
    generate_unit_quaternion,
    get_body_data,
    get_cartograhic_radians,
    get_illumination,
    get_lon_lat,
    get_n_colors,
    get_positions,
    get_quaternions,
)
from cesium_script.util.format import clean_utc


@pytest.fixture(scope="session", autouse=True)
def setup_spice():
    kernel_home = os.path.abspath(os.path.join(os.path.dirname(__file__), "kernels"))
    # Prepare the temporary metakernel
    mk_temporal = tempfile.NamedTemporaryFile(mode="w", delete=False)
    template = os.path.join(kernel_home, "juice_mini.tm")
    with open(template) as template_file, mk_temporal as mk_file:
        content = template_file.read()
        mk_file.write(content.replace("#HOME#", kernel_home))

    sp.kclear()
    sp.furnsh(mk_temporal.name)


def test_get_n_colors() -> None:
    color_list = get_n_colors(6)
    assert len(color_list) == 6


def test_get_body_data() -> None:
    radius, flatness = get_body_data("Earth")
    assert radius == pytest.approx(6378.1366)
    assert flatness == 0

    radius, flatness = get_body_data("FAKE_PLANET")
    assert radius == pytest.approx(10)
    assert flatness == 0


def test_get_lon_lat() -> None:
    lon, lat, alt = get_lon_lat([0, 1000, 0], 0, 100)
    assert lon == pytest.approx(math.pi / 2)
    assert lat == 0.0
    assert alt == pytest.approx(900)


def test_generate_cartesians() -> None:
    cesium_cartesians = generate_cartesians([[0, 0, 1], [0, 0, 2]])
    assert len(cesium_cartesians) == 6


def test_generate_cartesians_timed() -> None:
    cesium_cartesians = generate_cartesians([[0, 0, 1], [0, 0, 2]], 5)
    assert len(cesium_cartesians) == 8


def test_generate_unit_quaternion() -> None:
    quaternions = generate_unit_quaternion([[0, 0, 0, 1], [0, 0, 0, -1]], 5)
    assert len(quaternions) == 10


def test_generate_html_table() -> None:
    sample = {"key1": "value1"}
    html_content = generate_html_table(sample)
    assert len(html_content) == 78


def test_clean_utc() -> None:
    clean_utc_str = clean_utc("2023-05-09T00:00:00Z")
    assert clean_utc_str == "20230509T0000"


def test_get_positions() -> None:
    start = sp.utc2et("2030-10-15T00:00:00")
    ets = [start + step for step in range(0, 3600, 300)]
    positions = get_positions(ets, "JUICE", "JUPITER")
    assert len(positions) == 12

    cartographic = get_cartograhic_radians("JUPITER", positions)
    assert len(cartographic) == 12


def test_get_quaternions() -> None:
    start = sp.utc2et("2030-10-15T00:00:00")
    ets = [start + step for step in range(0, 3600, 300)]
    get_quaternions(ets, "IAU_JUPITER", "J2000")


def test_get_illumination() -> None:
    start = sp.utc2et("2030-10-15T00:00:00")
    ets = [start]
    positions = get_positions(ets, "JUICE", "JUPITER")
    get_illumination(start, "JUICE", "JUPITER", positions[0])
    with pytest.raises(NotFoundError):
        get_illumination(start, "FAKE", "JUPITER", positions[0])
